import logging
import hmac
import hashlib
import http.client
import urllib.parse
import ssl
from utils import make_request

DEVICE_ID_FILE = '/etc/deviceid'
DEVICE_TOKEN_FILE = '/etc/device_token'

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logHandler = logging.StreamHandler()
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
logHandler.setFormatter(formatter)
logger.addHandler(logHandler)

logger.setLevel(logging.DEBUG)

class DeviceManager:
    def __init__(self):
        self.deviceId = self.readDeviceId()
        self.device_token = self.read_device_token()

    def save_device_token(self, token):
        try:
            with open(DEVICE_TOKEN_FILE, 'w', encoding='utf-8') as file:
                file.write(token)
            logger.debug(f"Device token saved: {token}")
            self.device_token = token
        except Exception as e: 
            logger.error(f"Failed to save device token {e}")

    def read_device_token(self):
        try: 
            with open(DEVICE_TOKEN_FILE, 'r', encoding='utf-8') as file:
                token = file.read().strip()
                logger.debug(f"Read device token: {token}")
                return token
        except FileNotFoundError:
            logger.warning("Device token file not found")
            return ""
        except Exception as e:
            logger.error(f"Failed to read device token: {e}")
            return ""

    def validate_device_token(self, token):
        try:
            url = 'https://teddy-api-production-333008702573.us-east4.run.app/api/health'

            headers = {
                'X-Authorization': f'Bearer {token}',
                'User-Agent': 'DeviceClient/1.0'
            }

            context = ssl.create_default_context()

            status, data = make_request(url, headers=headers, context=context)

            if status == 200:
                logger.debug("Device token is valid")
                return True
            elif status is None:
                raise ConnectionError("Failed to connect to validation server")
            else:
                logger.debug(f"Device token validation failed with status code: {status}")
                return False
        except Exception as e:
            logger.error(f"Error validating device token: {e}")
            raise
    
    def readDeviceId(self):
        try:
            with open(DEVICE_ID_FILE, 'r', encoding="utf-8") as file:
                device_id = file.read().strip()
                logger.debug("Read device ID: %s", device_id)
                return device_id
        except FileNotFoundError:
            logger.error("Error: The file %s was not found.", DEVICE_ID_FILE)
            return ""
        except IOError:
            logger.error("Error: Unable to read the file %s.", DEVICE_ID_FILE)
            return ""
        except Exception as e:
            logger.error("An unexpected error occurred: %s", str(e))
            return ""
