import logging
import urllib.parse
import http.client
import ssl

# Set up logging
logger = logging.getLogger(__name__)

def make_request(url, headers=None, context=None, max_redirects=5):
    redirects = 0
    while redirects < max_redirects:
        parsed_url = urllib.parse.urlparse(url)

        # Establish the HTTPS connection
        conn = http.client.HTTPSConnection(
            parsed_url.hostname,
            parsed_url.port or 443,
            context=context,
            timeout=5
        )

        # Send the GET request
        conn.request("GET", parsed_url.path or "/", headers=headers)

        # Get the response
        response = conn.getresponse()

        # Read and decode the response data
        data = response.read().decode('utf-8')

        # Log the status and data
        logger.debug(f"Response status: {response.status}")
        logger.debug(f"Response data: {data}")

        # Handle redirects
        if response.status in (301, 302, 303, 307, 308):
            location = response.getheader('Location')
            if not location:
                logger.error("Received redirect status code but no 'Location' header found.")
                break
            logger.info(f"Redirecting to {location}")
            url = urllib.parse.urljoin(url, location)
            redirects += 1
            conn.close()
            continue
        else:
            # Not a redirect status code
            return response.status, data

    logger.error("Too many redirects")
    return None, None